#include "Particle.hpp"

Particle::Particle() {
    position = ofVec2f(ofRandom(ofGetWidth()), ofRandom(ofGetHeight()));
    force = ofVec2f(ofRandom(-50, 50), ofRandom(-50, 50));
    baseSize = 1;
    
    screenWidth = ofGetWidth();
    screenHeight = ofGetHeight();
}

void Particle::update(float &speedMultiplier) {
    sineShake *= 0.999;
    sineFrequency *= 0.999;
    
    if(useGravity) {
        force += ofVec2f(0, 100 * ofGetLastFrameTime());
    }
    
    ofVec2f averagePoint = ofVec2f();
    for(auto &oneFriend : friends) {
        averagePoint += oneFriend->position;
    }
    averagePoint /= friends.size();
    
    if(followFriends) {
        for(auto &oneFriend : friends) {
            ofVec2f delta = oneFriend->position - position;
            
            float originalLength = force.length();
            force += delta * 0.1 * ofGetLastFrameTime();
            force = force.normalize() * originalLength;
            
            if(useOnlyFirstFriend) break;
        }
    }
    
    ofVec2f directionToAverage = ofVec2f(averagePoint.x - position.x, averagePoint.y - position.y);
    
    float angleDelta = force.angle(directionToAverage);
    
    if(rotateTowardsCenterOfFriends && averagePoint != ofVec2f()) {
        force.rotate(angleDelta * 0.01);
    }

    position += force * speedMultiplier * ofGetLastFrameTime();
    
    if(disabledIfGoesOut && (
       	position.x < 0 ||
    	position.x > ofGetWidth() ||
        position.y < 0 ||
        position.y > ofGetHeight()
       )){
        isActive = false;
    }
    
    if(position.x < 0) position = ofVec2f(position.x + screenWidth, position.y);
    if(position.x > screenWidth) position = ofVec2f(position.x - screenWidth, position.y);
    
    if(position.y < 0) position = ofVec2f(position.x, position.y + screenHeight);
    if(position.y > screenHeight) position = ofVec2f(position.x, position.y - screenHeight);
}

void Particle::draw(float &particleSize) {
    if(!isActive) return;
    
    ofVec2f cross = force;
    cross.rotate(90).normalize();
    ofVec2f currentPosition = position + cross * sin(ofGetElapsedTimef()*sineFrequency) * sineShake;
    
    ofSetColor(255, 255, 255);
    ofDrawCircle(currentPosition, baseSize);
    
    if(particleSize > 3) {
        for (int i = 0; i < 5; i++) {
            ofSetColor(255, 255, 255, 1.f/10*255);
            ofDrawCircle(currentPosition, i*3);
        }
    }
}

void Particle::findFriend(){
    if(allParticles == nullptr) return;
    if(allParticles->size() <= friends.size() + 1) return;
    
    int randomIndex = 0;
    while(true) {
        randomIndex = rand() % allParticles->size();
        if(randomIndex != index) break;
    }
    
    auto newFriend = &allParticles->at(randomIndex);
    friends.push_back( newFriend );
}

void Particle::clearFriends() {
    friends.clear();
}

void Particle::shake(){
    sineShake = 10;
    sineFrequency = 5;
}
