#include "ofApp.h"
#include <algorithm>

void ofApp::setup(){
    soundplayer.load("dark_sunday_01.mp3");
    
    ofSetBackgroundColor(20);
    ofSetBackgroundAuto(false);

    addTimer(0.1, [&]() {
        addParticles(15);
        soundplayer.play();
    });
    addTimer(calculateBeatToSeconds(1,0), [&]() {
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = true;
        }
    });
    addTimer(calculateBeatToSeconds(2,0), [&]() {
        addParticles(90);
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = true;
        }
    });
    
    addTimer(calculateBeatToSeconds(4,0), [&]() {
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = false;
            particle.followFriends = true;
            particle.useOnlyFirstFriend = true;
        }
    });
    addTimer(calculateBeatToSeconds(5,0), [&]() {
        for(auto &particle : particles) {
            particle.shake();
        }
    });
    
    addTimer(calculateBeatToSeconds(8,0), [&]() {
        speedMultiplier = 3;
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = false;
            particle.followFriends = false;
            particle.useOnlyFirstFriend = false;
        }
    });
    
    addTimer(calculateBeatToSeconds(9,0), [&]() {
        speedMultiplier = 1;
    });
    addTimer(calculateBeatToSeconds(10,0), [&]() {
        particles.clear();
        addParticles(15);
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = true;
        }
    });
    
    addTimer(calculateBeatToSeconds(11,0), [&]() {
        addParticles(200);
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = true;
            particle.followFriends = true;
            particle.useOnlyFirstFriend = true;
        }
    });
    
    addTimer(calculateBeatToSeconds(12,0), [&]() {
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = false;
            particle.followFriends = true;
            particle.useOnlyFirstFriend = false;
            particle.useGravity = true;
        }
    });
    
    for(int beatI = 12; beatI < 14; beatI++) {
        for(int beatStep = 0; beatStep < 3; beatStep++) {
            float beatStepTime = ((float)beatStep*2.f+2.f)/8.f;
            addTimer(calculateBeatToSeconds(beatI, beatStepTime), [&]() {
                for(auto &particle : particles) {
                    particle.force = ofVec2f(particle.force.x, -100);
                }
            });
        }
    }
    
    addTimer(calculateBeatToSeconds(15,0), [&]() {
        particles.clear();
        addParticles(100);
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = true;
            particle.followFriends = false;
            particle.useOnlyFirstFriend = false;
        }
    });
    
    addTimer(calculateBeatToSeconds(17,0), [&]() {
        particles.clear();
        addParticles(100);
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = true;
            particle.followFriends = true;
        }
    });
    
    addTimer(calculateBeatToSeconds(18,0), [&]() {
        addParticles(300);
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = true;
            particle.followFriends = true;
        }
    });
    
    addTimer(calculateBeatToSeconds(20,0), [&]() {
        particles.clear();
        addParticles(30);
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = true;
            particle.followFriends = true;
        }
    });
    
    addTimer(calculateBeatToSeconds(22,0), [&]() {
        for(auto &particle : particles) {
            particle.rotateTowardsCenterOfFriends = false;
            particle.followFriends = false;
            particle.disabledIfGoesOut = true;
        }
    });
    
    addTimer(175, [&]() {
        soundplayer.stop();
        started = false;
        ended = true;
        for(auto &timer : timers) {
            timer.targetTimeReached = false;
            timer.currentTime = 0;
        }
        cout << "playing ended!" << endl;
    });
}


void ofApp::update(){
    if(ended) {
        for(auto &particle : particles) {
            float speed = 1;
            particle.update(speed);
        }
    }
    
    
    if(started == false) return;
    if(soundplayer.isPlaying() == false) {
        soundplayer.play();
    }
    
    spectrum = ofSoundGetSpectrum(audioSpectrumBands);
    
    particleSize = 1 + spectrum[1]*spectrum[1] * 10; //clap
    tailLength = spectrum[4]; //swoosh
    
    //skip the first one on purpose
    float tempSum = 0;
    for(int i=1;i<audioSpectrumBands;i++){
        tempSum += spectrum[i];
    }
    spectrumSum = max(0.f, min(1.f, tempSum / 2));
    
    float speed = (1 + spectrumSum*spectrumSum * 20) * speedMultiplier;
    for(auto &particle : particles) {
        particle.update(speed);
    }
    
    for(auto &timer : timers) {
        timer.update();
    }
}


void ofApp::draw(){
    ofSetColor(0, 0, 0, 100 * tailLength);
    ofDrawRectangle(0, 0, ofGetWidth(), ofGetHeight());


    
    for(auto &particle : particles) {
        particle.draw(particleSize);
    }
    
    if(ofGetElapsedTimef() > 2*60 + 55) {
        ofSetColor(0, 0, 0, 5);
        ofDrawRectangle(0, 0, ofGetWidth(), ofGetHeight());
    }
}


void ofApp::addParticles(int count) {
    for(int i = 0; i < count; i++) {
        addParticle();
    }
    refreshFriends();
}

void ofApp::addParticle(){
    particles.push_back(Particle());
    
    Particle * particle = &particles[particles.size()-1];
    particle->index = particles.size()-1;
    particle->allParticles = &particles;
}

void ofApp::refreshFriends(){
    for(auto &particle : particles) {
        particle.clearFriends();
        particle.findFriend();
        particle.findFriend();
        particle.findFriend();
    }
}

void ofApp::keyPressed(int key) {
    if(started == false) {
        particles.clear();
    	started = true;
    }
}

void ofApp::addTimer(float targetTime, std::function<void()> onTime){
    timers.push_back(Timer());
    
    Timer * timer = &timers[timers.size()-1];
    timer->targetTime = targetTime;
    timer->onComplete = onTime;
}

float ofApp::calculateBeatToSeconds(int beat, float step) {
    float timeShift = 0;
    float bmp = 112.5;
    return ((float)beat + step)*4.f * bmp / 60.f + timeShift;
}
